/*----------------------------------------------------File: image.c-----------*/
/******************************************************************************/
/*                                                                            */
/*                                                                            */
/*                              Vartalaap                                     */
/*----------------------------------------------------------------------------*/
/*                  Multimedia Multiparty  Conferencing System                */
/*                                                                            */
/*                                                                            */
/*                  Written by:VIJAY TALATI                                   */
/*                             ERNet Lab,                                     */
/*                             Department of Computer Science,                */
/*                             Indian Institute of Technology, Bombay 400076. */
/*                             E-mail : vijay@gateway.iitb.ernet.in           */
/*                                                                            */
/******************************************************************************/


#include	<X11/Xlib.h> 
#include	<X11/Xutil.h>
#include	<X11/cursorfont.h>
#include	<xview/xview.h>
#include	<xview/frame.h>
#include	<xview/panel.h>
#include	<xview/openmenu.h>
#include	<xview/textsw.h>
#include	<xview/canvas.h>
#include	<xview/xv_xrect.h>
#include	<xview/scrollbar.h>
#include	<xview/cursor.h>

#include 	"vartalaap.h"
#include 	"posdefs.h"

#include 	"../common/talklib.h"
#include 	"../common/protodefs.h"

Panel_setting	filename_proc() ;
void			process_image() ;
void			draw_image() ;

void			event_proc()  ;
void			repaint_proc() ;
void			get_image_format() ;
void			CreateXImage() ;
void			FloydDitherize1 () ;
void			FloydDitherize8 () ;
void			FSDither() ;

int				piv_x, piv_y;
int				currentX, currentY ;
char			DrawingRectOn = FALSE ;
char			DrawingCircOn = FALSE ;
extern	unsigned long col_pixel[] ;


/*----------------------------------------------------process_image ()--------*/
void	process_image( item, event)
Panel_item	item ;
Event	*event;
{
	int	grp_id ;

	/* If filename frame already opened then return */
	if (filename_frame != NULL)
		return ;

	/* Get group id */
	grp_id = find_group( event) ;

	/* Check for maximum allowed displayed images */
	if ( number_of_displayed_images[grp_id] == MAX_IMAGES-1 )
	{
		sprintf(errstr,"%d (MAX ) images. Cannot display more",
		number_of_displayed_images[grp_id]) ;
		errmsg( errstr) ;
		return ;
	}

	/* Create the dialog sub frame to get filename */
	filename_frame = (Frame) xv_create(grp_box_ptr[grp_id], FRAME,
						XV_HELP_DATA,	"VARTALAAP:imagefilename",
						XV_HEIGHT,      IFNAME_HEIGHT, 
						XV_WIDTH,       IFNAME_WIDTH, 
						XV_X,		xv_get(grp_box_ptr[grp_id], XV_X)
											+GBOX_WIDTH/2 -20,
						XV_Y,		xv_get(grp_box_ptr[grp_id], XV_Y)+60,
						FRAME_LABEL,    "FILE NAME PROMPT",
                        NULL );
 
	filename_panel = (Panel)xv_create(filename_frame, PANEL,
						XV_HELP_DATA,	"VARTALAAP:imagefilename",
                        XV_X,   0,
                        XV_Y,   xv_get(filename_frame, XV_HEIGHT)-20,
                        NULL ) ;

	xv_set( filename_frame,	XV_SHOW, TRUE, NULL) ;
	(void) xv_create(filename_panel, PANEL_TEXT,
                PANEL_LABEL_STRING, "Image File Name:",
                PANEL_VALUE_DISPLAY_LENGTH,     IFNAME_DISPLAY_LEN,
				PANEL_CLIENT_DATA,	grp_id ,
                PANEL_NOTIFY_PROC, filename_proc,
                NULL );
	
	xv_set( filename_frame,	XV_SHOW, TRUE, NULL) ;
}

/*----------------------------------------------------filename_proc ()--------*/
Panel_setting	filename_proc( item, event)
Panel_item	item ;
Event	*event ;
{

	char		username[MAXPATHLEN] ;
	char 		*filename ;
	int			grp_id ;
   	Cursor    	cursor;
   	Server_image svr_image;
	int			filetype ;

	Window		win ;
	int			cpid ;
	FILE		*fp ;

	switch(event_action(event)) {

	case '\n':
	case '\r':
	case '\033':
		filename = (char *)xv_get(item, PANEL_VALUE) ;
		grp_id = (int) xv_get( item, PANEL_CLIENT_DATA) ;

		/* Defining the watch cursor */
		win = xv_get(xv_get(filename_panel, CANVAS_NTH_PAINT_WINDOW,0),
			XV_XID) ;
		XDefineCursor( theDisplay, win, watch) ;
		XFlush(theDisplay) ;

		xv_destroy_safe( filename_frame) ;
		/*Check if file exists fro reading */
		if ( (fp= fopen( (char *)filename, "r") )== NULL )
		{
			sprintf(errstr," @@Cannot open %s for reading ", filename) ;
			errmsg(errstr) ;
			filename_frame = NULL ;/*WORKHERE*/ /* Free the filename frame*/
			return ;
		}
		fclose( fp) ;

		/* First send the imagesend request to server and check if allowed*/
		if (check_image_channel( grp_id) != FREE)
		{
			errmsg("Image channel busy") ;
			return ;
		}

		/* Set filename_frame to NULL. Destroy doesn't seem to
		be doing it */
		filename_frame = NULL ;
	
		/* Create the image for local screen */
		if ( create_image(grp_box_ptr[grp_id], grp_id , CurrUsername, 
					filename, &filetype, FALSE) != -1)
		{
			transfer_image(event, grp_id, filetype) ;
		}
		break ;

	default :
		return(panel_text_notify(item, event)) ;
	}
}

/*-----------------------------------------------------create_image ()--------*/
int		create_image( base, grp_id,  username, filename, ftype,HistImage)
Frame	base ;
int		grp_id;
char	*username ;
char	*filename ;
int		*ftype ;
int		HistImage ;/*Flag to indicate if this is for history */
{

	FILE	*fp ;
	Window	xwin ;
	char	bm_name [MAX_LINE_LEN];
	char	label[MAX_LINE_LEN] ;
	int		filetype ;
	int		pos_x, pos_y ;

	/*Set global group id . Not neat !!!*//*WORKHERE*/ 
	group_id = grp_id ;

	/* Check if file is readable */
	strcpy( img_fname, filename ) ;
	if ( (fp= fopen( (char *)img_fname, "r") )== NULL )
	{
		sprintf(errstr," @@Cannot open %s for reading ", img_fname) ;
		errmsg(errstr) ;
		return(-1) ;
	}
	fclose( fp) ;

	/* Find Out the format of the image file */	
	get_image_format( filename, &filetype) ;
	(*ftype) = filetype ;
	if ( filetype  == UNKNOWN )
	{
		sprintf(errstr,"Unknown format of image file : %s", img_fname) ;
		errmsg(errstr) ;
		return(-1) ;
	}

	CreateXImage(filename, filetype) ;

	/* Append the username */
	if (HistImage)
		sprintf(label,"HISTORY Image [Sent by: %s]", username) ;
	else
		sprintf(label,"Image [Sent by: %s]", username) ;
 
	/*Get the index into the Image array */
	img_index = find_free_rec(grp_id) ;
	if ( img_index == -1)
		return(-1) ;

	/* Determine the location for the image frame, such that it does
		not overlap the grpwindow and also, it shd not go outside the screen*/
	get_pos( &pos_x, &pos_y, grp_id, img_index) ;
 
	Image[grp_id][img_index].image_frame = (Frame) xv_create( 
		base, FRAME,
		XV_HELP_DATA,	"VARTALAAP:imageframe",
		XV_HEIGHT,	/*IFRM_HEIGHT, */ pHIGH,
		XV_WIDTH,	/*IFRM_WIDTH, */ pWIDE,
		XV_X,		pos_x ,
		XV_Y,		pos_y ,
		FRAME_LABEL,	label,
		NULL) ;

	Image[grp_id][img_index].image_canvas = (Canvas) xv_create( 
		Image[grp_id][img_index].image_frame, CANVAS,
		CANVAS_REPAINT_PROC,	draw_image,
		CANVAS_X_PAINT_WINDOW,	TRUE,
		XV_HEIGHT,	pHIGH,
		XV_WIDTH,	pWIDE ,
		CANVAS_AUTO_SHRINK, FALSE,
		CANVAS_AUTO_EXPAND, TRUE,
		NULL ) ;

	if (HistImage == FALSE)
	{
	(void) xv_set(canvas_paint_window(
		Image[grp_id][img_index].image_canvas),
		WIN_EVENT_PROC,	event_proc,
		WIN_CONSUME_EVENTS,	KBD_DONE, KBD_USE, LOC_DRAG, LOC_MOVE,
				LOC_WINENTER, LOC_WINEXIT, WIN_ASCII_EVENTS, 
				WIN_MOUSE_BUTTONS, NULL,
		NULL) ;
	}
	else
	{
	(void) xv_set(canvas_paint_window(
		Image[grp_id][img_index].image_canvas),
		WIN_CONSUME_EVENTS,	KBD_DONE, KBD_USE, LOC_DRAG, LOC_MOVE,
				LOC_WINENTER, LOC_WINEXIT, WIN_ASCII_EVENTS, 
				WIN_MOUSE_BUTTONS, NULL,
		NULL) ;
	}

	xwin = xv_get( canvas_paint_window(
					Image[grp_id][img_index].image_canvas),
					 XV_XID) ;

	/* Creating the GCs and pixmaps reqd . */
	Image[grp_id][img_index].imggc= XCreateGC(theDisplay, xwin, 0, NULL) ;
	/*XSetForeground( theDisplay, Image[grp_id][img_index].imggc, 1) ;*/
	/*XSetBackground( theDisplay, Image[grp_id][img_index].imggc, 0) ;*/
	XSetForeground( theDisplay, Image[grp_id][img_index].imggc, White) ;
	XSetBackground( theDisplay, Image[grp_id][img_index].imggc, Black) ;

	/* The Mark GC is also created here */
	Image[grp_id][img_index].markgc= XCreateGC(theDisplay, xwin, 0, NULL) ;
	/*XSetForeground( theDisplay, Image[grp_id][img_index].markgc, 1) ;*/
	/*XSetBackground( theDisplay, Image[grp_id][img_index].markgc, 0) ;*/
	XSetForeground( theDisplay, Image[grp_id][img_index].markgc, Black) ;
	XSetBackground( theDisplay, Image[grp_id][img_index].markgc, White) ;
	/* Set in XOR mode */
	XSetFunction( theDisplay, Image[grp_id][img_index].markgc, 0x06) ;
	/*XSetFunction( theDisplay, Image[grp_id][img_index].markgc, GXXor) ;*/
	/* Set the line attr */
	XSetLineAttributes( theDisplay, Image[grp_id][img_index].markgc, 2,
			LineSolid, CapProjecting, JoinMiter) ;

	/* Create the Pixmap to store the bitmap */
	Image[grp_id][img_index].imgpix = XCreatePixmap( theDisplay, xwin, 40,40,XDefaultDepth(theDisplay,0));

	xv_set(Image[grp_id][img_index].image_canvas, XV_SHOW, TRUE, NULL ) ;
	xv_set(Image[grp_id][img_index].image_frame, XV_SHOW, TRUE, NULL ) ;

}

/*-----------------------------------------------------CreateXImage ()--------*/
void	CreateXImage( fname, ftype)
char	*fname ;
int	ftype ;
{
	int	nc ;

	switch ( ftype)
	{
		case XBM :
			/* Create the image to be displayed */
			/* WORKHERE: Check for errors */
			LoadXBM( fname) ; 

			break ;
		case PBM:
			LoadPBM(fname, nc) ;
			break ;
		case GIF :
			/* Create the image to be displayed */
			LoadGIF( fname, nc) ;

			break ;
		case JFIF:
			/* Create the image to be displayed */
			LoadJFIF( fname, nc) ;
			break ;

		default :
			/* WORKHERE
			sprintf(errstr,"Unsupported format ") ;
			errmsg(errstr ) ;
			*/
			break ;
	}

}


/*-----------------------------------------------------draw_image ()----------*/
void	draw_image( canvas, paint_window, dpy, xwin, xrects)
Canvas	canvas;
Xv_window	paint_window;
Display	*dpy;
Window	xwin ;
Xv_xrectlist	*xrects ;
{

	unsigned	char *imagedata ;
	unsigned char *idata, *ip, *pp, *lip ;
	int		half ;
	int	x,y ;
	int i,j; 
	int	DispDepth ;
	int	nullCount , imWIDE ;

	static int	flag =0;

	/* Flag for avoiding doing things on second call to draw_image */
	if (flag == 1)
	{
		flag=0 ;
		return ;
	}
	else
		flag = 1;


	xv_set( xv_get( canvas, CANVAS_NTH_PAINT_WINDOW, 0),
		WIN_CURSOR,		pen_cursor,
		NULL) ; 

/* this creation and allocation of memory has to move out : WORKHERE*/

	DispDepth = XDefaultDepth( theDisplay, XScreenNumberOfScreen(theScreen)) ;
	if ( DispDepth == 1)
	{
		theImage = XCreateImage( theDisplay,
			XDefaultVisualOfScreen(XDefaultScreenOfDisplay(theDisplay)), 
			DispDepth, XYPixmap, 0, NULL, pWIDE, pHIGH, 8, 0) ;

		if (!theImage) printf("Error CreateImage \n" ) ;

		theImage->data = (char *) malloc(pHIGH*pWIDE*sizeof(char)) ; 
		/*theImage->data = (char *) pic ;*/

		FloydDitherize1( theImage) ;
	}
	else
	if ( DispDepth == 8)
	{

		SortColormap() ;
		AllocColors() ;

		nullCount = ( 4 - (pWIDE % 4)) & 0x03 ;
		imWIDE = nullCount + pWIDE ;
		imagedata = (unsigned char *)Malloc(imWIDE*pHIGH*sizeof(char)) ;
		if (!imagedata) printf("Error .. memory allocation \n") ;

		for (i=0, pp=pic, lip=imagedata; i<pHIGH; i++ )
		{
			for (j=0 ; j<pWIDE; j++,pp++,lip++)
				*lip = (unsigned char ) col_pixel[*pp];
				/**lip = (unsigned char ) def_xcol[*pp].pixel ;*/
			for ( j=0; j<nullCount ; j++, lip++)
				*lip = 0 ;
		}
	
		theImage = XCreateImage( theDisplay,
			XDefaultVisualOfScreen(XDefaultScreenOfDisplay(theDisplay)), 
			DispDepth, ZPixmap, 0, (char *)imagedata, pWIDE, pHIGH, 32,imWIDE) ;

		if (!theImage) printf("Error CreateImage \n" ) ;
		/*FloydDitherize8( theImage) ;*/
	}
	else
	if (DispDepth == 4)
	{
		SortColormap() ;
		AllocColors() ;
		theImage = XCreateImage( theDisplay,
            XDefaultVisualOfScreen(XDefaultScreenOfDisplay(theDisplay)),
            DispDepth, ZPixmap, 0, NULL, pWIDE, pHIGH, 32, 0) ;
 
        if (!theImage) printf("Error CreateImage \n" ) ;
         
        idata= (unsigned char *) malloc(pHIGH*pWIDE*sizeof(char)) ; ;
        theImage->data = idata ;

		for (i=0, pp=pic, lip=idata; i<pHIGH; i++, lip+=4)
		{
			for (j=0, ip=lip , half=0; j<pWIDE; j++,pp++,half++)
			{
				if (ImageByteOrder(theDisplay) == LSBFirst)
				{
					if ( half&1) 
					{
						*ip = *ip + ((col_pixel[*pp]&0x0f)<<4); 
						ip++;
					}
					else
					{
						*ip = col_pixel[*pp]&0x0f ;
					}
				}
				else
				{
					if (half&1)
					{
						*ip = *ip + (col_pixel[*pp]&0x0f);
						ip++;
					}
					else
						*ip = (col_pixel[*pp]&0x0f) << 4 ;
				}
			}
		}
	
        /*FloydDitherize8( theImage) ;*/
    }

	XPutImage(theDisplay, xwin, Image[group_id][img_index].imggc, theImage,0,0,
	   0,0, pWIDE, pHIGH) ;		

	number_of_displayed_images[group_id]++ ;
	Image[group_id][img_index].flag = DISPLAYED ;
/*FreeAllColors() ;*/
	XFlush(theDisplay) ;
}



/*-----------------------------------------------------event_proc ()----------*/
void		event_proc( window, event)
Xv_Window	window ;
Event		*event ;
{
	int		grp_id ;
	Window	win ;
	int 	b,h ;

	win = xv_get (window, XV_XID) ; 
	grp_id = group_id ;

	if (event_is_ascii(event))
		;
	else
		switch(event_action(event) ) 	
		{
			case LOC_MOVE:
			case LOC_DRAG:
				/* If the mouse if moved after clicking of
				   left or middle button, then a rubber band
				   square is tobe drawn over the image*/
				if ((DrawingRectOn == TRUE) || (DrawingCircOn == TRUE) ) 
				{
					find_img_index( event, &grp_id, &img_index) ;
					calculate_size( grp_id, img_index, &piv_x, &piv_y, &b, &h) ;
					
					if ( DrawingRectOn == TRUE)
						XDrawRectangle( theDisplay, win, 
							Image[grp_id][img_index].markgc, piv_x,piv_y,b, h) ;
					else
						XDrawArc( theDisplay, win, Image[grp_id][img_index].markgc,
							piv_x, piv_y, b, h, 0, 64*360) ;

					Image[grp_id][img_index].currentX = event_x(event) ;
					Image[grp_id][img_index].currentY = event_y (event) ;

					calculate_size( grp_id, img_index, &piv_x, &piv_y, &b, &h) ;

					if ( DrawingRectOn == TRUE)
						XDrawRectangle( theDisplay, win, 
							Image[grp_id][img_index].markgc, piv_x,piv_y,b, h) ;
					else
						XDrawArc( theDisplay, win, Image[grp_id][img_index].markgc,
						piv_x,piv_y,b, h, 0, 360*64) ;

				}
					
				break ;

			case ACTION_SELECT:
			case MS_LEFT:
				/* Clicking of Left button first starts the
				   rubber band rect. Second click ends it and
				   Xmits the co-ords of the rect */

				if (DrawingRectOn == FALSE)
				{
					find_img_index( event, &grp_id, &img_index) ;
					DrawingRectOn = TRUE ;
					Image[grp_id][img_index].currentX = event_x(event) ;
					Image[grp_id][img_index].currentY = event_y(event) ;
					Image[grp_id][img_index].lastX 
						= Image[grp_id][img_index].currentX ;
					Image[grp_id][img_index].lastY 
						= Image[grp_id][img_index].currentY ;
					calculate_size( grp_id, img_index, &piv_x, &piv_y, &b, &h) ;
					XDrawRectangle( theDisplay, win, Image[grp_id][img_index].markgc, 									piv_x,piv_y,b, h) ;
				}
				else
				{
					find_img_index( event, &grp_id, &img_index) ;
					DrawingRectOn = FALSE ;
					/* Send the mark message */
					switch( T_image_mark( grp_id, img_index+StartImgIndx[grp_id],
						Image[grp_id][img_index].lastX,
						Image[grp_id][img_index].lastY,
						Image[grp_id][img_index].currentX, 
						Image[grp_id][img_index].currentY,
						 RECT) )
					{
                       	case    _NoError:
                       		break;
 
                        case    _GroupNotJoined:
                        	 errmsg("Group not joined");
                           	 break;
 
                        case    _NoSuchGroup:
                        	 errmsg("No such group ");
                           	 break;
 
                        case    _OtherFailure:
                        	 errmsg("General Failure");
                           	 break;
 
                        default:
                        	 errmsg("invalid value ");
                           	 break;
 
                	}
				}

				break ;

			case ACTION_ADJUST:
			case MS_MIDDLE:
				{
				/* Clicking of middle button first starts the
				   rubber band circ. Second click ends it and
				   Xmits the co-ords of the circ */

				if (DrawingCircOn == FALSE)
				{
					find_img_index( event, &grp_id, 
							&img_index) ;
					DrawingCircOn = TRUE ;
					Image[grp_id][img_index].currentX = event_x(event) ;
					Image[grp_id][img_index].currentY = event_y(event) ;
					Image[grp_id][img_index].lastX 
					= Image[grp_id][img_index].currentX ;
					Image[grp_id][img_index].lastY 
					= Image[grp_id][img_index].currentY ;
					calculate_size( grp_id, img_index, &piv_x, &piv_y, &b, &h) ;
					XDrawArc( theDisplay, win, Image[grp_id][img_index].markgc,
						piv_x, piv_y, b, h, 0, 64*360) ;

				}
				else
				{
					find_img_index( event, &grp_id, &img_index) ;

					DrawingCircOn = FALSE ;
					/* Send the mark message */
					switch( T_image_mark( grp_id, img_index,
						Image[grp_id][img_index].lastX,
						Image[grp_id][img_index].lastY,
						Image[grp_id][img_index].currentX, 
						Image[grp_id][img_index].currentY,
						ARC) )
					{
                        		case    _NoError:
                                		break;
 
                        		case    _GroupNotJoined:
                               			 errmsg("Group not joined");
                               			 break;
 
                        		case    _NoSuchGroup:
                               			 errmsg("No such group ");
                               			 break;
 
                        		case    _OtherFailure:
                               			 errmsg("General Failure");
                               			 break;
 
                        		default:
                               			 errmsg("invalid value ");
                               		 break;
 
                			}
				}


				break ;	
				}

			case ACTION_MENU:
			case MS_RIGHT:
				break ;

			default :
				return ;
		}

}


/*---------------------------------------------------repaint_proc ()----------*/
void	repaint_proc(canvas, paint_window, dpy, xwin, xrects)
Canvas			canvas;
Xv_window		paint_window;
Display			*dpy;
Window			xwin;
Xv_xrectlist	*xrects ;
{

	int	b, h ;
	int	grp_id = group_id ;

	/* Getting positive width and height */
        b = Image[grp_id][img_index].currentX-Image[grp_id][img_index].lastX ;
        if ( b < 0 )
        {
                b = b * (-1.0) ;
                piv_x = Image[grp_id][img_index].currentX ;
        }
        else
                piv_x = Image[grp_id][img_index].lastX ;
 
        
        h = Image[grp_id][img_index].currentY-Image[grp_id][img_index].lastY ;
        if ( h < 0 )
        {
                h = h * (-1.0) ;
                piv_y = Image[grp_id][img_index].currentY ;
        }
        else
                piv_y = Image[grp_id][img_index].lastY ;
 
}


/*
 * create_cursor() creates a bull's eye cursor and assigns it
 * to the window (parameter).
 */
create_cursor(window)
Xv_Window window;
{
    Xv_Cursor      cursor;
    Server_image   image;
    Pixmap         pixmap;
    GC             gc;
    XGCValues      gcvalues;

    image = (Server_image)xv_create(XV_NULL, SERVER_IMAGE,
        XV_WIDTH,       16,
        XV_HEIGHT,      16,
        NULL);
    pixmap = (Pixmap)xv_get(image, XV_XID);
    /* Create GC with reversed foreground and background colors to
     * clear pixmap first.  Use 1 and 0 because pixmap is 1-bit deep.
     */
    gcvalues.foreground = 0;
    gcvalues.background = 1;
    gc = XCreateGC(theDisplay, pixmap, GCForeground|GCBackground, &gcvalues);
    XFillRectangle(theDisplay, pixmap, gc, 0, 0, 16, 16);
    /*
     * Reset foreground and background values for XDrawArc() routines.
     */
    gcvalues.foreground = 1;
    gcvalues.background = 0;
    XChangeGC(theDisplay, gc, GCForeground | GCBackground, &gcvalues);
    XDrawArc(theDisplay, pixmap, gc, 2, 2, 12, 12, 0, 360 * 64);
    XDrawArc(theDisplay, pixmap, gc, 6, 6, 4, 4, 0, 360 * 64);

    /* Creaste cursor and assign it to the window (parameter) */
    cursor = xv_create(XV_NULL, CURSOR,
        CURSOR_IMAGE,   image,
        CURSOR_XHOT,    7,
        CURSOR_YHOT,    7,
        NULL);
    xv_set(window, WIN_CURSOR, cursor, NULL);

    /* free the GC -- the cursor and the image must not be freed. */
    XFreeGC(theDisplay, gc);
}

/*-----------------------------------------------------get_image_format-------*/
void	get_image_format( fname, ftype)
char	*fname ;
int		*ftype ;
{
	FILE	*fp ;
	unsigned char	magicno[8] ; /* First 8 bytes of file */

	if ((fp = fopen(fname,"r")) == NULL)
	{
		return ;
	}
	
	fread( magicno, 8, 1, fp) ;
	fclose(fp) ;

	(*ftype) = UNKNOWN ;

	if (strncmp(magicno,"GIF87a",6)==0 ||
		strncmp(magicno,"GIF89a",6)==0) (*ftype) = GIF ;

	else if (strncmp(magicno,"VIEW", 4) == 0 ||
		strncmp(magicno,"WEIV", 4) ==0) (*ftype) = PM ;

	else if (magicno[0] == 'P' && magicno[1] >='1' &&
		magicno[1] <= '6') (*ftype) = PBM ;

	else if (strncmp( magicno, "#define", 7)==0) (*ftype) = XBM ;

	else if ( magicno[0] == 0x59 && (magicno[1]&0x7f) == 0x26 &&
		magicno[2]==0x6a && (magicno[3]&0x7f)==0x15) (*ftype) = SUNRAS ;

	else if (magicno[0] == 0xff && magicno[1] == 0xd8 &&
		magicno[2]==0xff) (*ftype) = JFIF ;
	
	else
		(*ftype) = UNKNOWN ;

}
/************************/
void FloydDitherize1(ximage)
     XImage *ximage;
{
  /* same as FloydDitherize8, but output is a 1-bit per pixel XYBitmap,
     packed 8 pixels per byte */

  register short *dp;
  register byte   pix8, bit;
  short          *dithpic;
  int             i, j, err, bperln, order;
  byte           *pp, *image, w1, b1, w8, b8, rgb[256];

  int		white =0;
  int		black =1;



  /* first thing to do is build rgb[], which will hold the B/W intensity
     of the colors in the r,g,b arrays */
  for (i=0; i<256; i++)
    rgb[i] = MONO(r[i], g[i], b[i]);


  image  = (byte *) ximage->data;
  bperln = ximage->bytes_per_line;
  order  = ximage->bitmap_bit_order;

/* 
  if (DEBUG) fprintf(stderr,"Ditherizing1...");
*/
 
  dithpic = (short *) malloc(pWIDE * pHIGH * sizeof(short));
  if (dithpic == NULL) printf("not enough memory to ditherize");
 
  w1 = white&0x1;  b1=black&0x1;
  w8 = w1<<7;  b8 = b1<<7;        /* b/w bit in high bit */
 
  /* copy rgb[pic] into dithpic so that we can run the algorithm */
  pp = pic;  dp = dithpic;
  for (i=pHIGH * pWIDE; i>0; i--) *dp++ = fsgamcr[rgb[*pp++]];

  dp = dithpic;
  pp = image;

  for (i=0; i<pHIGH; i++) {
    pp = image + i*bperln;


    if (order==LSBFirst) {
      bit = pix8 = 0;
      for (j=0; j<pWIDE; j++,dp++) {
        if (*dp<128) { err = *dp;     pix8 |= b8; }
                else { err = *dp-255; pix8 |= w8; }

        if (bit==7) {
          *pp++ = pix8;  bit=pix8=0;
        }
        else { pix8 >>= 1;  bit++; }

        if (j<pWIDE-1) dp[1] += ((err*7)/16);

        if (i<pHIGH-1) {
          dp[pWIDE] += ((err*5)/16);
          if (j>0)       dp[pWIDE-1] += ((err*3)/16);
          if (j<pWIDE-1) dp[pWIDE+1] += (err/16);
        }
      }  
      if (bit) *pp++ = pix8>>(7-bit);  /* write partial byte at end of line */
    }

    else {   /* order==MSBFirst */

     bit = pix8 = 0;
      for (j=0; j<pWIDE; j++,dp++) {
        if (*dp<128) { err = *dp;     pix8 |= b1; }
                else { err = *dp-255; pix8 |= w1; }

        if (bit==7) {
          *pp++ = pix8;  bit=pix8=0;
        }
        else { pix8 <<= 1; bit++; }

        if (j<pWIDE-1) dp[1] += ((err*7)/16);

        if (i<pHIGH-1) {
          dp[pWIDE] += ((err*5)/16);
          if (j>0)       dp[pWIDE-1] += ((err*3)/16);
          if (j<pWIDE-1) dp[pWIDE+1] += (err/16);
        }
      }  
      if (bit) *pp++ = pix8<<(7-bit);  /* write partial byte at end of line */
    }
  }  

/*
  if (DEBUG) fprintf(stderr,"done\n");
*/

  free(dithpic);
}
/*----------------------------------------------check_image_channel()---------*/
/*----------------------------------------------------------------------------*/
Status	check_image_channel( grp_id)
int		grp_id ;
{
	/* Sent the image send request to the server*/


	switch(T_sendimagereq( grp_id))
	{

		case _ImageReqRejected: 
				return( BUSY) ;
				break ;

		case _NoError:
				return( FREE) ;
				break ;

		default :
				/*errstr("Unknown reply for image req") ;*/
				break ;
	}

	return(BUSY) ;
}





/************************/
void FloydDitherize8(image)
     byte *image;
{
  /* takes epic, and builds a black&white dithered version of it.
     stores result in 8bit Pixmap format in 'image' */

  int i;
  byte *p;
  int		white =0;
  int		black =1;

  FSDither(pic, pWIDE, pHIGH, image);

  /* set to 'black' and 'white' instead of '0' and '1' */
  if (black != 0 || white != 1) {
    for (i=pWIDE*pHIGH, p=image; i>0; i--, p++) {
      if (*p) *p = white;  else *p = black;
    }
  }
}

/************************/
void FSDither(inpic, w, h, outpic)
byte *inpic, *outpic;
int   w,h;
{
  /* takes inpic, and builds a black&white dithered version of it.
     stores result as 1 byte per pixel format in 'image'
     black = 0;  white = 1;
     temporarily mallocs a w*h array of SHORTS.
     (need to be signed, also to avoid overflow problems.)  */

  /* floyd-steinberg dithering.
   *
   * ----   x    7/16
   * 3/16  5/16  1/16
   *
   */

  short *dp, *dithpic;
  int    i, j, err, w1, h1;
  byte  *pp, rgb[256];

  /*if (DEBUG) fprintf(stderr,"Ditherizing...");*/

  /* first thing to do is build rgb[], which will hold the B/W intensity
     of the colors in the r,g,b arrays */
  for (i=0; i<256; i++)
    rgb[i] = MONO(r[i], g[i], b[i]);


  dithpic = (short *) malloc(w*h * sizeof(short));
  /*if (dithpic == NULL) FatalError("not enough memory to ditherize");*/

  w1 = w-1;  h1 = h-1;

  /* copy rgb[inpic] into dithpic so that we can run the algorithm */
  pp = inpic;  dp = dithpic;
  for (i=w*h; i>0; i--) *dp++ = fsgamcr[rgb[*pp++]];

  dp = dithpic;  pp = outpic;
  for (i=0; i<h; i++) {
    /*if ((i&15) == 0) WaitCursor();*/
    for (j=0; j<w; j++,dp++,pp++) {
      if (*dp<128) { err = *dp;     *pp = 0; }
              else { err = *dp-255; *pp = 1; }

      if (j<w1) dp[1] += ((err*7)/16);

      if (i<h1) {
        dp[w] += ((err*5)/16);
        if (j>0)  dp[w1] += ((err*3)/16);
        if (j<w1) dp[w+1] += (err/16);
      }
    }
  }

  /*if (DEBUG) fprintf(stderr,"done\n");*/

  free(dithpic);
}
/*-----------------------------------------------------End of image.c---------*/
