/*----------------------------------------------------File: utils.c-----------*/
/******************************************************************************/
/*                                                                            */
/*                                                                            */
/*                              Vartalaap                                     */
/*----------------------------------------------------------------------------*/
/*                  Multimedia Multiparty  Conferencing System                */
/*                                                                            */
/*                                                                            */
/*                  Written by:VIJAY TALATI                                   */
/*                             ERNet Lab,                                     */
/*                             Department of Computer Science,                */
/*                             Indian Institute of Technology, Bombay 400076. */
/*                             E-mail : vijay@gateway.iitb.ernet.in           */
/*                                                                            */
/******************************************************************************/

#include	<X11/Xlib.h> 
#include	<X11/Xutil.h>
#include	<xview/xview.h>
#include	<xview/frame.h>
#include	<xview/panel.h>
#include	<xview/openmenu.h>
#include	<xview/textsw.h>
#include	<xview/canvas.h>
#include	<xview/xv_xrect.h>
#include	<xview/scrollbar.h>

/*
 *	Utility functions usable by all files
 */
#include	"vartalaap.h"
#include	"posdefs.h"
#include	"../common/inc.h"

#include	"logo.h"

extern	int	LocalPort ;
extern	int	ServerPort ;

void	draw_logo() ;
void	process_send() ;
void	find_img_index() ;
void	get_bitmap_name() ;
void	getlogname() ;
void	quit_hist() ;

void	InitSpline () ;

short	logo_bits[] = {
#include "varta_icon.h"
} ;

/* From xv */
/**
unsigned	char	*pic ;
int	pWIDE, pHIGH ;
unsigned	char	r[256], g[256], b[256] ;
**/
/*
 *	Command line parsing
 */

/*--------------------------------------------------Usage ()--------*/
void	Usage()
{

	fprintf(stderr, "Usage: vartalaap ");
	fprintf(stderr, "    [-u<user-name>]\n");
	fprintf(stderr, "    [-b]\n");
	fprintf(stderr, "    [-p<port>]\n");
	exit(1);

}


/*--------------------------------------------------ScanArgs ()--------*/
void	ScanArgs(argc,argv)
	int argc;
	char **argv;
{

	/* Scan Argument list */

	while(--argc > 0)
	 {
		++argv;
		 if(**argv == '-')
			switch(*++*argv)
			{
				case 'u':
					strcpy(CurrUsername, ++*argv) ;
					printf("User Name : %s \n",
						CurrUsername ) ;	
					break ;

				case 'p': /*Server port */
					ServerPort = atoi(++*argv) ;
					if (ServerPort <0)
						ServerPort = 0 ;
					printf("ServerPort : %d \n",
						ServerPort ) ;	
					break ;

				case 'b':
					BBoardMode = TRUE ;
					break ;

				default:
					fprintf(stderr,"Unknown Option %c\n", 
							**argv);
					Usage();
					break;

			}
		else
			Usage();

	}/* End of while */

}

/*--------------------------------------------------show_logo ()--------*/
void	show_logo()
{
	Canvas	logo_canvas ;

	
	/*Create canvas in the base frame */
	logo_canvas = (Canvas) xv_create ( base_frame, CANVAS,
			XV_X,	0,
			XV_Y,	0,
			CANVAS_REPAINT_PROC,	draw_logo ,
			CANVAS_X_PAINT_WINDOW,	TRUE,
			CANVAS_AUTO_EXPAND,	TRUE,
			CANVAS_AUTO_SHRINK,	TRUE,
			NULL ) ;

	xv_set( logo_canvas,	XV_SHOW, TRUE, NULL );
}

/*--------------------------------------------------draw_logo ()--------*/
void	draw_logo( canvas, paint_window, dpy, xwin, xrects)
Canvas	canvas ;
Xv_window	paint_window ;
Display	*dpy ;
Window	xwin ;
Xv_xrectlist	*xrects ;
{

	GC	logogc ;	
	Pixmap	logopix ;

	unsigned long	fg, bg ;
	unsigned int	depth =1;
	fg = (long) 1 ;
	bg = (long) 0 ;

	logogc = XCreateGC( theDisplay, xwin, 0, NULL );
	/*XSetForeground(theDisplay, logogc, 1) ;*/
	/*XSetBackground(theDisplay, logogc, 0) ;*/
	XSetForeground(theDisplay, logogc, Black) ;
	XSetBackground(theDisplay, logogc, White) ;

	logopix = XCreatePixmap(theDisplay, xwin, 40,40, 1) ;
	
	if ( (logopix = XCreatePixmapFromBitmapData( theDisplay, xwin, logo,
		logo_width, logo_height, fg, bg, depth) ) != NULL )
	{
		xv_set( canvas, XV_HEIGHT, logo_height, NULL );
		xv_set( canvas, XV_WIDTH, logo_width, NULL) ;
	
		XCopyPlane( theDisplay, logopix, xwin, logogc, 0, 0, logo_width, 
			logo_height, 0, 0, 0x01) ;
		XFlush( theDisplay) ;
	}

	/*XSetForeground(theDisplay, logogc, 0) ;*/
	XSetForeground(theDisplay, logogc, White) ;
	/*XSetBackground(theDisplay, logogc, 1) ;*/
	XSetBackground(theDisplay, logogc, Black) ;

	XDrawString( theDisplay, xwin, logogc, logo_width/2-10,logo_height/2+40 , "A", 
			strlen("A")) ;
	XDrawString( theDisplay, xwin, logogc, logo_width/2-30,logo_height/2+60 , 
			"MultiParty", strlen("MultiParty")) ;
	XDrawString( theDisplay, xwin, logogc, logo_width/2-30,logo_height/2+80 , 
			"MultiMedia", strlen("MultiMedia")) ;
	XDrawString( theDisplay, xwin, logogc, logo_width/2-65,logo_height/2+100 , 
			"Conferencing System", strlen("Conferencing System")) ;
	XDrawString( theDisplay, xwin, logogc, logo_width/2-120,logo_height/2+120 , 
			"Copyright: ERNET, CSE, IIT Bombay 1993", 
			strlen("Copyright: ERNET, CSE, IIT Bombay 1992")) ;
	XFlush( theDisplay) ;
}


 
/*--------------------------------------------------getllogname ()--------*/
/*
 * getlogname : Fetches the users name form the environment variable LOGNAME
 */
void	getlogname( str)
char    *str ;
{
        strcpy(str , (char *)getenv("LOGNAME") );
}
	

/*--------------------------------------------------calculate_size ()--------*/
/* Calculating the width, height etc for mark of image */
void	calculate_size( grp_id, index, px, py, breadth, height)
int	grp_id, index ;
int	*px, *py, *breadth, *height ;
{
	(*breadth) = Image[grp_id][index].currentX-Image[grp_id][index].lastX ;
        if ( (*breadth) < 0 )
        {
               (*breadth) = -(*breadth) ;
                (*px) = Image[grp_id][index].currentX ;
        }
        else
                (*px) = Image[grp_id][index].lastX ;


         (*height) = Image[grp_id][index].currentY-Image[grp_id][index].lastY ;
         if ( (*height) < 0)
         {
                 (*height) = -(*height) ;
                 (*py) = Image[grp_id][index].currentY ;
         }
         else
                 (*py) = Image[grp_id][index].lastY ;
}

/*--------------------------------------------------find_img_index ()--------*/
void 	find_img_index( event, group_id, index)
Event	*event ;
int	*group_id ;
int	*index ;
{
	int	i, j;

	for ( i=0; i< MAX_GRP_NO; i++)
	{
		for (j=0; j< MAX_IMAGES; j++)
			if ( Image[i][j].image_canvas != NULL )
			if (event->ie_win == canvas_paint_window(Image[i][j].image_canvas) )
			{
				(*group_id) = i ;
				(*index) = j ;
			}
	}
}


/*--------------------------------------------------get_bitmap_name ()--------*/
/* Routine to get the bitmap name from the bitmap file */

void	get_bitmap_name( fname, s)
char	*fname ;
char	*s ;
{
	
	FILE	*fp ;
	char	c ;
	char	*loc ;

	fp = fopen( fname, "r") ;	
	if (fp == NULL)
	{
		strcpy(s, fname) ;
		return ;	/*Cannot read the file, so set the filename as
				 the bitmap name */
	}

	/*Assuming that the bitmap files will always have a line of the
	  form '#define	name_width', we get name from there. (This is 
	  true atleast for standard bitmap files */

	while((c=fgetc(fp)) !=EOF)
	{
		ungetc(c, fp) ;
		/* Get the line from the file */
		fgets(s, MAX_LINE_LEN-2, fp) ;
		loc = (char *)strdup(s) ;
		/* Get the first word from the line */
		strtok(s, " "); 	
		/*Is is "#define" ? */
		if ( strcmp(s,"#define") == 0 )
		{
			/* Extract name part from it */
			strtok(loc,"_") ;
			strcpy(s,strchr(loc,' ') );
			fclose(fp) ;
			return  ;	
		}
	}
	fclose(fp) ;
	strcpy(s, fname) ;
	return ;
}

/*Grabbed from xv */
/*********************/
double EvalSpline(xa,ya,y2a,n,x)
double y2a[],x;
int n,xa[],ya[];
{
  int klo,khi,k;
  double h,b,a;

  klo = 0;
  khi = n-1;
  while (khi-klo > 1) {
    k = (khi+klo) >> 1;
    if (xa[k] > x) khi = k;
    else klo = k;
  }
  h = xa[khi] - xa[klo];
  if (h==0.0) printf("bad xvalues in splint\n");
  a = (xa[khi]-x)/h;
  b = (x-xa[klo])/h;
  return (a*ya[klo] + b*ya[khi] + ((a*a*a-a)*y2a[klo] +(b*b*b-b)*y2a[khi])
          * (h*h) / 6.0);
}
   
 
/*********************/
void GenerateFSGamma()
{
  /* this function generates the Floyd-Steinberg gamma curve (fsgamcr)

     This function generates a 4 point spline curve to be used as a
     non-linear grey 'colormap'.  Two of the points are nailed down at 0,0
     and 255,255, and can't be changed.  You specify the other two.  If
     you specify points on the line (0,0 - 255,255), you'll get the normal
     linear reponse curve.  If you specify points of 50,0 and 200,255, you'll
     get grey values of 0-50 to map to black (0), and grey values of 200-255
     to map to white (255) (roughly).  Values between 50 and 200 will cover
     the output range 0-255.  The reponse curve will be slightly 's' shaped. */

  int i,j;
  static int x[4] = {0,32,224,255};
  static int y[4] = {0, 0,255,255};
  double yf[4];

  InitSpline(x, y, 4, yf);

  for (i=0; i<256; i++) {
    j = (int) EvalSpline(x, y, yf, 4, (double) i);
    if (j<0) j=0;
    else if (j>255) j=255;
    fsgamcr[i] = j;

  }
}
/*********************/
void InitSpline(x,y,n,y2)
     int *x, *y, n;
     double *y2;
{
  /* given arrays of data points x[0..n-1] and y[0..n-1], computes the
     values of the second derivative at each of the data points
     y2[0..n-1] for use in the splint function */

  int i,k;
  double p,qn,sig,un,u[MAX_GHANDS];

  y2[0] = u[0] = 0.0;

  for (i=1; i<n-1; i++) {
    sig = ((double) x[i]-x[i-1]) / ((double) x[i+1] - x[i-1]);
    p = sig * y2[i-1] + 2.0;
    y2[i] = (sig-1.0) / p;
    u[i] = (((double) y[i+1]-y[i]) / (x[i+1]-x[i])) -
           (((double) y[i]-y[i-1]) / (x[i]-x[i-1]));
    u[i] = (6.0 * u[i]/(x[i+1]-x[i-1]) - sig*u[i-1]) / p;
  }  
  qn = un = 0.0;
 
  y2[n-1] = (un-qn*u[n-2]) / (qn*y2[n-2]+1.0);
  for (k=n-2; k>=0; k--)
    y2[k] = y2[k]*y2[k+1]+u[k];
}    
 
void	get_pos( xp, yp, gid, imgid)
int	*xp, *yp ;
int	gid ;
int	imgid ;
{
	/*Try to avoid overlapping the grpwindow and also avoid to display
	image outside the screen */
	
	if ( ( xv_get(grp_box_ptr[gid],XV_X) + xv_get(grp_box_ptr[gid],XV_WIDTH)
			+ 20 ) >= MAX_WIDTH_OF_SCREEN )
	{
	/* grp_window is to the very right of the screen, display image to 
		the left*/
		(*xp) = xv_get(grp_box_ptr[gid],XV_X)-(img_index+1)*10 - pWIDE;
		if ((*xp) <0 )
			(*xp) = 0 ;/*Necessary ?? */
	}
	else
	{
		
		(*xp) = xv_get(grp_box_ptr[gid],XV_X)+(img_index+1)*10 
				+ xv_get(grp_box_ptr[gid], XV_WIDTH) ;;
		if ((*xp) >MAX_WIDTH_OF_SCREEN )
			(*xp) = 0 ;/*Necessary ?? */
	}

	(*yp) = xv_get(grp_box_ptr[gid], XV_Y) +30 +(img_index+1)*20 ; 

	if (((*yp)+pHIGH)  > MAX_HEIGHT_OF_SCREEN)
		(*yp) = 20 ;

}

/*----------------------------------------------------------sigint_func()-----*/
Notify_value	sigint_func( client, sig, when)
Notify_client	client;
int	sig;
Notify_signal_mode	when ;
{

	int		no_of_grp , i;

	/* Quit the session gracefully*/
 
	/* Leave all the groups */
	for (no_of_grp = 0 ; no_of_grp < MAX_GRP_NO; no_of_grp++ )
		if ( usr_grp_list[no_of_grp].group_id != -1)
		
			for(i=0; i<MAX_USERS; i++){
				textsw_reset( UserWin[no_of_grp][i].txt_win, 0,0) ;
            T_leave(usr_grp_list[no_of_grp].group_id) ;
		}
 
	
	xv_destroy_safe( client) ;
	exit (0) ;
}
		
/*--------------------------------------------------quit_hist ()----*/ 
void    quit_hist(item, event)
Panel_item	item;
Event		*event;
{
	int	grp_id ;
	int	i ;

	/* Find group id from window id */

	grp_id = find_group_from_hist_win( event) ;

	for (i=0; i<MAX_USERS; i++)
	{
		textsw_reset( HistWin[grp_id][i].txt_win, 0, 0) ;

		xv_set(HistWin[grp_id][i].txt_win, XV_HEIGHT, 
				UWC_HEIGHT, 	
				NULL) ;
	}
	xv_set(hist_frame[grp_id], XV_HEIGHT, 
				UWC_HEIGHT, 	
				NULL) ;
	xv_set(hist_frame[grp_id], XV_SHOW, FALSE) ;
	
	/*xv_destroy_safe( hist_frame[grp_id]) ;*/
	/*hist_frame[grp_id] = NULL ;*/
}


/*--------------------------------------------------End of file: utils.c-----*/
