/*-----------------------------------------------------File: vartalaap.h------*/
/******************************************************************************/
/*									      */
/*									      */
/*				Vartalaap				      */
/*----------------------------------------------------------------------------*/
/*                  Multimedia Multiparty  Conferencing System                */
/*									      */
/*	Written by:							      */
/*			Yezdezard Lashkari				      */
/*			Viswanath Ramachandran				      */
/*			Sanjay Malpani					      */
/*      Modified by:                                                          */
/*                      Vijay Talati                                          */
/*                      ERNet Lab.                                            */
/*									      */
/*									      */
/*		Department of Computer Science and Engineering		      */
/*		Indian Institute of Technology, Bombay 400076.		      */
/*									      */
/******************************************************************************/

/*
*	This file contains definitions of data structures required by
*	program VARTALAAP
*/
#include <stdio.h>
#include <errno.h>
#include <sys/param.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/cursorfont.h>
#include <xview/xview.h>
#include <xview/frame.h>
#include <xview/panel.h>
#include <xview/openmenu.h>
#include <xview/textsw.h>
#include <xview/canvas.h>
#include <xview/xv_xrect.h>
#include <xview/scrollbar.h>
#include <xview/server.h>
#include <xview/seln.h>
#include <xview/notice.h>
#include <xview/cursor.h>
#include <xview/font.h>

/* The bitmap file containing the logo */
/*#include "logo"*/

/*------------------------------------------------------definitions-----------*/

#define	ARBIT		10140
#define	ON 		1
#define	OFF 		0	
#define	RETURN_KEY	13	

#define	USED	1
#define	UNUSED	0

#define	GRP_NAME_LENGTH 255	/* Length of group name */
#define	MAX_MSG_NO 	50	/* Max. number of messages */

#define	FIRST		1
#define	NOT_FIRST	0 

#define	MAX_USERS	5
#define	MAX_IMAGES	20	/* Max number of images a user can have 
				simultanously*/
#define	MAX_CHARS	0 	/* Max number of chars after which the 
				   data will be send. Also sent on CR */
#define	 VNGROUPS	3	
#define	MAX_GRP_NO 	VNGROUPS	/* Maximum no of group allowed to join */
#define	MESSAGE_SIZE 	1000	/* Maximum message size that can be sent */
#define	MAX_AUDIO_LEN 	2000	/* Maximum allowed audio in bytes.  */

#define	MAX_PKT_SIZE	8000	/* Maximum chunk of data that can be written
					at a time to socket */

#define	AUDIO_BUF_SZ	4000

#define	NAME_LENGTH 	50		/* Max lenth of user name */
#define	RWIN 		2		/* Receive window */
#define	SWIN 		1		/* Send window */

#define	FREE		0
#define	BUSY		1

#define	NOT_DISPLAYED	0
#define	DISPLAYED	1

#define	NONE		-1
#define MAX_ENTRIES	4
#define MAKENEW		0
#define JOINGRP		1	
#define LEAVEGRP 	2	
#define HISTORY 	3	
#define UNAME 		4	

/* The image file formats defs */
#define	UNKNOWN		0
#define	GIF		1
#define	PM		2
#define	PBM		3
#define	XBM		4
#define	SUNRAS		5
#define	JFIF		6

#define TITLE		"VARTALAAP" /* Main window name */

#define	TAG_FONT_STRING	"LucidaSans-BoldItalic"

#define	ARC	0
#define	RECT	1

#define MAX_LINE_LEN	100	
typedef	unsigned char	byte;

/*--------------------------------------------------Global variables----------*/


int	NO_OF_GRP;	/* No of group user has joied */
int 	option;		/* User option is taken in this variable */
int	debug_mode ;	/* Debug switch */
int	BBoardMode ;

int	count ;

struct usr_grp_type {
	int	group_id;	/* Id of the group */
	}; 

struct usr_grp_type  usr_grp_list[MAX_GRP_NO];

char	message[MESSAGE_SIZE];	/* Space to store messages */
char	user_name[NAME_LENGTH];	/* Array to store user name */
struct	msgpair	*messagelist;
int	HistoryGroup ;
int	LocalPort ;

/* The array of pointers to the group boxes (widgets) */
Frame	grp_box_ptr [MAX_GRP_NO] ;
Frame	hist_frame [MAX_GRP_NO] ;
Panel	hist_panel [MAX_GRP_NO] ;
Textsw	snd_textsw [MAX_GRP_NO] ;
Textsw	rcv_textsw [MAX_GRP_NO] ;

int	recv_proc_id [MAX_GRP_NO] ;

Display 	*theDisplay ;
Screen		*theScreen ;
Visual		*theVisual ;
Colormap	theCmap ;
Window		basewin ;
unsigned int	ncells ;
int			disp_depth ;
int			ncols ;
int			mono ;

Frame	base_frame;
Frame	filename_frame;
Frame	cmd_frame ;
Frame	base_frame ;
Frame	dial_frame;
Panel	dial_panel ;
Panel	filename_panel;
Panel	btn_panel[MAX_GRP_NO] ;

int	group_id ;
int	DrawingFlag ;
int	prev_strlen[MAX_GRP_NO] ; /*Used to get send data */

/* The structure for user windows*/
struct gbox {
		int	user_id ;
		int	State ;
		char	uname[NAME_LENGTH] ;
		char	pfname[MAXPATHLEN] ;
		Canvas	info_canvas ;
		GC		info_gc ;
		Textsw	txt_win ;
		} UserWin[MAX_GRP_NO][MAX_USERS], HistWin[MAX_GRP_NO][MAX_USERS] ;

int		StartTxtPos[VNGROUPS][MAX_USERS] ;
int		StartImgIndx[VNGROUPS];
struct	{
		int	win;/* Send or receive widnow?*/
		int	pos;/* Position of message  from start pos*/
	}location[MAX_GRP_NO][MAX_MSG_NO] ;

int	ph_flag[MAX_GRP_NO] ;	

char	errstr[80] ;

int	img_index ;		/* Index inot the Image array */

/* The structure which contains all data structs. related to an Image */
struct {
	int	flag ;			/*Flag showing whether image displayed*/
	Pixmap	imgpix ;
	GC	imggc ;
	GC	markgc ;
	int	lastX ;
	int	lastY;
	int	currentX ;
	int	currentY; 
	Canvas	image_canvas ;
	int	width ;
	int	height ;
	int	xhot ;
	int	yhot ;
	Frame	image_frame ;
	}Image [VNGROUPS][MAX_IMAGES] ;

int	number_of_displayed_images[VNGROUPS] ; /* number of images each
						group has displayed */

/* Black and white pixel*/
unsigned	long	Black ;
unsigned	long	White ;

int		CurrGroupId ; 
int		AudioStop[VNGROUPS] ;


/* The Frame for history */
Frame	hist_base[MAX_GRP_NO] ;
Textsw	hist_textsw[MAX_GRP_NO] ;

/* The Frame for userlist */
Frame	ulist_base[MAX_GRP_NO] ;
Textsw	ulist_textsw[MAX_GRP_NO] ;
Canvas	ulist_canvas[MAX_GRP_NO] ;
Panel	ulist_panel[MAX_GRP_NO] ;
int		ulist_displayed[MAX_GRP_NO] ;

char	CurrUsername[NAME_LENGTH] ;
char	*UserPhoto ;
int	photosize ;
Xv_server	server ;
 
char	img_fname[MAXPATHLEN] ;

char	userlist[1000] ;
int		ppath[MAXPATHLEN] ;

/*Cursors */
Cursor	hand1;
Cursor	hand2;
Cursor	pencil;
Cursor	watch;

/** Globals reqd for images : from XV */
unsigned char	*pic ;
int	pWIDE,pHIGH ;
unsigned char r[256],g[256], b[256] ;
unsigned char fsgamcr[256];

/*Globals for cursor */
Xv_Cursor       pen_cursor ;
Server_image    pen_svr_image ;
 
XImage	*theImage ;
/* MONO returns total intensity of r,g,b components */
#define MONO(rd,gn,bl) (((rd)*11 + (gn)*16 + (bl)*5) >> 5)  /*.33R+ .5G+ .17B*/

#define	MAX_GHANDS	16 /* maximum# of GRAF handles */

int numcols ;
unsigned char	rdisp[256], gdisp[256], bdisp[256] ;
float normaspect ;

#define F_FORMAT	1
#define	F_GIF		0
#define	F_COLORS	0
#define	F_FULLCOLOR	0

XColor	def_xcol[256] ;

/*RANGE forces a to be in the range b..c (inclusive) */
#define	RANGE(a,b,c) { if (a<b) a=b; if (a>c) a=c;}


/*AUDIO*/
int	Audio_fd ;

/*----------------------------------------------------End of vartalaap.h------*/
