/*------------------------------------------------------init.c----------------*/
/******************************************************************************/
/*                                                                            */
/*                                                                            */
/*                                Vartalaap                                   */
/*----------------------------------------------------------------------------*/
/*                     Multimedia Conferencing Software                       */
/*                                                                            */
/*                                                                            */
/*        Written by:                                                         */
/*                        Yezdezard Lashkari                                  */
/*                        Viswanath Ramachandran                              */
/*                        Sanjay Malpani                                      */
/*                 and                                                        */
/*                        Vijay Talati                                        */
/*                                                                            */
/*                                                                            */
/*                Department of Computer Science and Engineering              */
/*                Indian Institute of Technology, Bombay 400076.              */
/*                                                                            */
/******************************************************************************/

#include	"../common/inc.h"
#include	"../common/protodefs.h"

#define		EXTERN	/* Null string */

#define		MAINFILE

#include	"clientlib.h"

static	int	padlength[4] = { 0, 3, 2, 1 };/*To pad to multiple of 4 bytes */

extern		int	LocalPort ;
extern		char *getenv();
extern		char *Malloc();
extern		long	getpid();

/*---------------------------------------------------SendConnPrefix()---------*/
void	SendConnPrefix(fd, username, photodata, psize, authproto, authstr)
FD		fd;
char	*username;
char	*photodata;
int		psize;
char	*authproto;
char	*authstr;
{
	LL_ConnClientPrefix llccp;
	int					i,lenblock ;
	char				*pBuf;
	int					endian;	/* Determine whether MSB first or LSB first */
	char				*clientinfo;

	ENTER_PROC("SendConnPrefix");

	/* To determine byte order */
	/*WORKHERE*/		
	endian = 1;
	if(*(char *)&endian)
		llccp.byteOrder = 'l';
	else
		llccp.byteOrder = 'B';

	if(authproto != (char *)NULL) 
		llccp.nBytesAuthProto 	= htons(strlen(authproto));
	else
		llccp.nBytesAuthProto	= htons(0);

	if(authstr != (char *)NULL) 
		llccp.nBytesAuthString	= htons(strlen(authstr));
	else
		llccp.nBytesAuthString	= htons(0);

		llccp.nBytesUserPhoto	= htonl(psize);

	if(username != (char *)NULL) 
		llccp.nBytesUserName	= htons(strlen(username));
	else
		llccp.nBytesUserName	= 0;


	llccp.pid = htonl((unsigned long)L_pid);

	/* Now compose the block to send to local server */

	lenblock = sizeof(LL_ConnClientPrefix) + 
			((ntohs(llccp.nBytesAuthProto) + 3) & ~3) +
			((ntohs(llccp.nBytesAuthString) + 3) & ~3) +
			((psize+ 3) & ~3) +
			((ntohs(llccp.nBytesUserName) + 3) & ~3); 

	/* The extra length to pad it to a multiple of 4 bytes */

	clientinfo = (char *)Malloc(lenblock);

	bcopy((char *)&llccp, clientinfo, sizeof(LL_ConnClientPrefix));
	pBuf = clientinfo + sizeof(LL_ConnClientPrefix);

	/* Now copy the authorisation protocol info*/
	bcopy(authproto, pBuf, ntohs(llccp.nBytesAuthProto));
	pBuf += ntohs(llccp.nBytesAuthProto);

	i = padlength[ntohs(llccp.nBytesAuthProto)& 3];
	/* pad */
	while(--i >= 0)
		*pBuf++ = 0;

	/* Now copy the authorisation string*/
	bcopy(authstr, pBuf, ntohs(llccp.nBytesAuthString));
	pBuf += ntohs(llccp.nBytesAuthString) ;

	i = padlength[ntohs(llccp.nBytesAuthString) & 3];
	/* pad */
	while(--i >= 0)
		*pBuf++ = 0;

	/* Now copy the user name string*/
	bcopy(username, pBuf, ntohs(llccp.nBytesUserName));
	pBuf += ntohs(llccp.nBytesUserName);

	i = padlength[ntohs(llccp.nBytesUserName) & 3];
	/* pad */
	while(--i >= 0)
		*pBuf++ = 0;

	/* Now copy the user photo string*/
	bcopy(photodata, pBuf, psize);
	pBuf += psize ;

	i = padlength[psize & 3];
	while(--i >= 0)
		*pBuf++ = 0;

	/* Now write this to the server */
	WriteToServer(fd, lenblock, (char *)clientinfo);

	Free(clientinfo) ;

	return;

}

/*---------------------------------------------------ReadServerReply()--------*/
void	ReadServerReply(fd)
FD		fd;
{
	ConnSetupPrefix ccp;
	int				len; 
	char			*str;
	int				slen;

	ENTER_PROC("ReadServerReply");

	ReadBuffer(fd, (char *)&ccp, sizeof(ConnSetupPrefix));

	len = ccp.lengthReason ;

	switch(ccp.success)
	{
		case TRUE:
			break;

		case FALSE:
			printf("Client unauthorised to connect up\n");
			/* now read the reason string */
			slen = (len + 3) & ~3;
			str = (char*)Malloc(slen + 3);
			if(slen)
			{
				if(!ReadBuffer(fd, str,slen))
				{
					ERROR("Length error reading failure str");
					close(fd);
					return;
				}
				str[slen] = '\0';
				printf("Server Reason:%s\n",str);
			}
			break;

		default:
			/* printf("Unknown flag:%d\n",ccp.success); */
			break;

	 }

	return ;
}

	
	
/*---------------------------------------------------ConnectToServer()--------*/
FD	ConnectToServer(hostname, username, photodata, psize, authproto, authstr, 
				lservname)
char	*hostname ;
char	*username;
char	*photodata;
int		psize;
char	*authproto;
char	*authstr;
char	*lservname;
{
	struct	sockaddr_in	server ;
	char	name[MAXHOSTNAMELEN];
	char	fullname[MAXNAMESZ];

	char	*host_name ;
	struct	hostent	*hp ;
	register FD	sock;
	int		SPort ;


	ENTER_PROC("ConnectToServer");

	bzero((char *)&server, sizeof(server)) ;

    if( hostname == (char *)NULL || *hostname == '\0')
    {
        if((host_name = getenv("CONFSERVER")) == NULL)
            FATAL("No hostname for conference server");
    }          
    else
    {
        host_name = (char *)
            Malloc((strlen(hostname) + 2) *sizeof(char));
        strcpy(host_name,hostname);
    }
 
    fprintf(stderr,"Connecting to manager at %s\n",host_name);
 
	if((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0 )
	{
		perror("Opening Stream Socket\n");
		/*exit(1);*//*WORKHERE*/ 
		return(-1) ;/* indicating error */
	}
	/* else */
 
    if(( hp = gethostbyname(host_name)) == NULL)
    {
        perror("Invalid host");
        exit(-1);
    }
 

	server.sin_family       = AF_INET ;
    bcopy((char *)hp->h_addr, (char *)&server.sin_addr, hp->h_length);


	SPort = GetServerPort() ;
	server.sin_port = htons(SPort) ;

	if( connect( sock, (struct sockaddr *)&server, 
							sizeof( struct sockaddr_in )) < 0)
	{
		perror("Couldnt connect to socket\n");
		close( sock );
		/*exit(1);*//*WORKHERE*/ 
		return(-1) ;/* indicating error */
	}

	/* else */
	DEBUG_LEVEL1((stderr,"Connected to Stream socket\n"));

	(void)SendConnPrefix(sock, username, photodata, psize, authproto, authstr);
	(void)ReadServerReply(sock);
	return(sock);
}

/*---------------------------------------------------T_init_talk_lib()--------*/
/* 
 * 	This file contains the routine to initialize the client library.
 */

Status	T_init_talk_lib(name,photodata, psize, authproto,authstring,localservername)
char	*name;					/* user's name */
char	*photodata;					/* user's photo */
char	*authproto;				/* Authorisation protocol*/
int		psize;				/* Size of photo string*/
char	*authstring;			/* Authorisation string */
char	*localservername;		/* Name to connect to for UNIX CONN */
{
	char	*p;


	if((name == (char*) NULL) || (name[0] == '\0'))
	{
		if((p = (char *)getenv("LOGNAME")) == (char *)NULL)
			FATAL("No username found");
		/* else */
		strcpy(L_username,p);
	}
	else
		strcpy(L_username,name);
	
	/* get the process id to store */
	L_pid = getpid();
	
	if((L_sock = ConnectToServer( ServerName, L_username, photodata, psize, authproto, 
					authstring,localservername)) <= 0)
	{
		printf("Couldnt connect to local server");
		return(-1) ;
	}
	INIT_LIB_FLAG = TRUE ;

	return(_NoError);
}

/*---------------------------------------------------GetServerPort()----------*/
int	GetServerPort()
{
    int   globport;

    globport = (int )(SERVER_BASE_PORT + ServerPort) ;
    DEBUG_LEVEL1((stderr,"Global port no:%d\n", globport));

    return(globport);
}
/*---------------------------------------------------End of init.c------------*/
