/*--------------------------------------------------------------libio.c-------*/
/******************************************************************************/
/*                                                                            */
/*                                                                            */
/*                                Vartalaap                                   */
/*----------------------------------------------------------------------------*/
/*                     Multimedia Conferencing Software                       */
/*                                                                            */
/*                                                                            */
/*        Written by:                                                         */
/*                        Yezdezard Lashkari                                  */
/*                        Viswanath Ramachandran                              */
/*                        Sanjay Malpani                                      */
/*                 and                                                        */
/*                        Vijay Talati                                        */
/*                                                                            */
/*                                                                            */
/*                Department of Computer Science and Engineering              */
/*                Indian Institute of Technology, Bombay 400076.              */
/*                                                                            */
/******************************************************************************/

/*
 *	This file contains routines to startup the client library,
 *	connect to the local server, accept requests from
 *	the client , and initialise client library data
 *	structures
 */
#ifdef COMPILETHIS
#undef COMPILETHIS
#endif

#include	"../common/inc.h"
#include	"../common/protodefs.h"		/* For the protocol defs */

#define		EXTERN	extern

#include	"clientlib.h"


static	int	padlength[4] = {0, 3, 2, 1};

/*------------------------------------------------LL_SendMakeGroupReq()-------*/
/*
 *	This is to send a make a new group request 
 *	information to the local server
 */
void	LL_SendMakeGroupReq(fd)
FD	fd;
{
	LL_MakeGroupReq llmgr;
	int				lenblock;
	char			*libinfo;

	ENTER_PROC("LL_SendMakeGroupReq");

	llmgr.reqType = _MakeGroupReq;

	/* Now compose the block to send to local server */

	lenblock = sizeof(LL_MakeGroupReq);

	libinfo = (char *)Malloc(lenblock);

	bcopy((char *)&llmgr, libinfo, sizeof(LL_MakeGroupReq));

	/* Now write this to the local server */
	WriteToServer(fd, lenblock, (char *)libinfo);

	LEAVE_PROC("LL_SendMakeGroupReq");

	return;

}
/*------------------------------------------------LL_SendHistoryReq()---------*/
/*
 *	This is to send a history of a group request 
 *	information to the local server
 */
void	LL_SendHistoryReq(fd,id)
FD	fd;
int	id;
{
	LL_HistoryReq	llhr;
	int				lenblock ;
	char			*libinfo;

	ENTER_PROC("LL_SendHistoryReq");

	llhr.reqType = _HistoryReq;
	llhr.gid = id ;

	/* Now compose the block to send to local server */

	lenblock = sizeof(LL_HistoryReq);

	libinfo = (char *)Malloc(lenblock);

	bcopy((char *)&llhr, libinfo, sizeof(LL_HistoryReq));

	/* Now write this to the local server */
	DEBUG_LEVEL3((stderr,"Writing Req to Server "));
	WriteToServer(fd, lenblock, (char *)libinfo);

	LEAVE_PROC("LL_SendHistoryReq");

	return;

}
/*-----------------------------------------------LL_SendUserListReq()---------*/
/*
 *	This is to send a userlist of a group request 
 *	information to the local server
 */
void	LL_SendUserListReq(fd,id)
FD	fd;
int	id;
{
	LL_UserListReq	llulr;
	int				lenblock;
	char			*libinfo;

	ENTER_PROC("LL_SendUserListReq");

	llulr.reqType = _UserListReq;
	llulr.gid = id ;

	/* Now compose the block to send to local server */

	lenblock = sizeof(LL_UserListReq);

	libinfo = (char *)Malloc(lenblock);

	bcopy((char *)&llulr, libinfo, sizeof(LL_UserListReq));

	/* Now write this to the local server */
	/*DEBUG_LEVEL3(("Writing Req to Server "));*/
	WriteToServer(fd, lenblock, (char *)libinfo);

	LEAVE_PROC("LL_UserListReq");

	return;

}

/*-----------------------------------------------LL_SendAnnotateReq()---------*/
/*
 *	This is to send a annotate request 
 *	information to the server
 */
void	LL_SendAnnotateReq(fd,id, uid, from, to)
FD		fd;
int		id;
int		uid, from, to ;
{
	LL_AnnotateReq	llar;
	int				lenblock;
	char			*libinfo;

	ENTER_PROC("LL_SendAnnotateReq");

	llar.reqType = _AnnotateReq;
	llar.gid = id ;
	llar.userid = uid ;
	llar.From = htons(from );
	llar.To = htons(to );

	/* Now compose the block to send to local server */

	lenblock = sizeof(LL_AnnotateReq);

	libinfo = (char *)Malloc(lenblock);

	bcopy((char *)&llar, libinfo, sizeof(LL_AnnotateReq));

	/* Now write this to the local server */
	/*DEBUG_LEVEL3(("Writing Req to Server " ));*/
	WriteToServer(fd, lenblock, (char *)libinfo);

	LEAVE_PROC("LL_SendAnnotateReq");

	return;

}
/*----------------------------------------------LL_SendImagemarkReq()---------*/
/*
 *	This is to send a imagemark request 
 *	information to the local server
 */
void	LL_SendImagemarkReq(fd,id, imageindex, startx, starty, endx, endy,type)
FD		fd;
int		id;
int		imageindex;
int		startx, starty, endx, endy ;
int		type ;
{
	LL_ImagemarkReq	llimr;
	int				lenblock;
	char			*libinfo;

	ENTER_PROC("LL_SendImagemarkReq");

	llimr.reqType = _ImagemarkReq;
	llimr.gid = id ;
	llimr.ImgIndx = imageindex ;
	llimr.startx = htons(startx );
	llimr.starty = htons(starty );
	llimr.endx = htons(endx );
	llimr.endy = htons(endy );
	llimr.type = type ;

	/* Now compose the block to send to local server */

	lenblock = sizeof(LL_ImagemarkReq);

	libinfo = (char *)Malloc(lenblock);

	bcopy((char *)&llimr, libinfo, sizeof(LL_ImagemarkReq));

	/* Now write this to the local server */
	/*DEBUG_LEVEL3(("Writing Req to Server " ));*/
	WriteToServer(fd, lenblock, (char *)libinfo);

	LEAVE_PROC("LL_SendImagemarkReq");

	return;

}

/*--------------------------------------------------LL_JoinGroupReq()---------*/
/*
 *	This is to send join group request
 *	information to the local server
 */
void	LL_SendJoinGroupReq(fd,id)
FD		fd;
int		id;
{
	LL_JoinGroupReq	lljgr;
	int			lenblock;
	char		*libinfo;

	ENTER_PROC("LL_JoinGroupReq");

	lljgr.reqType = _JoinGroupReq;
	lljgr.gid = id;

	/* Now compose the block to send to local server */

	lenblock = sizeof(LL_JoinGroupReq);

	/* The extra length to pad it to a multiple of 4 bytes */

	libinfo = (char *)Malloc(lenblock);

	bcopy((char *)&lljgr, libinfo, sizeof(LL_JoinGroupReq));

	/* Now write this to the local server */
	WriteToServer(fd, lenblock, (char *)libinfo);

	LEAVE_PROC("LL_JoinGroupReq");

	return;

}
/*---------------------------------------------LL_SendLeaveGroupReq()---------*/
/*
 *	This is to send leave group request
 *	information to the local server
 */
void	LL_SendLeaveGroupReq(fd,id)
FD		fd;
int		id;
{
	LL_LeaveGroupReq	lllgr;
	int					lenblock;
	char				*libinfo;

	ENTER_PROC("LL_LeaveGroupReq");

	lllgr.reqType = _LeaveGroupReq;
	lllgr.gid = id;

	/* Now compose the block to send to local server */

	lenblock = sizeof(LL_LeaveGroupReq);

	/* The extra length to pad it to a multiple of 4 bytes */

	libinfo = (char *)Malloc(lenblock);

	bcopy((char *)&lllgr, libinfo, sizeof(LL_LeaveGroupReq));

	/* Now write this to the local server */
	WriteToServer(fd, lenblock, (char *)libinfo);

	LEAVE_PROC("LL_LeaveGroupReq");

	return;

}

/*-----------------------------------------------LL_SendSendDataReq()---------*/
/*
 *	This is to send a data send request  
 *	information to the local server
 */
void	LL_SendSendDataReq(fd,id,data,nbytes)
FD		fd;
int		id;
char	*data;
int		nbytes;			/* No of bytes of data */
{
	LL_SendDataReq	llsdr;
	int				i,lenblock;
	char			*pBuf;
	char			*libinfo;
	int				bytes ;
	

	ENTER_PROC("LL_SendDataReq");

	llsdr.reqType = _SendDataReq;
	llsdr.gid = id;

	if( nbytes != strlen(data))
	{
		ERROR("No of bytes passed is wrong:Taking min value");
		DEBUG_LEVEL3((stderr,"strlen retd:%d\tnbytes passed:%d\n",
						strlen(data), nbytes));
		bytes = (strlen(data) < nbytes )? strlen(data):nbytes;
	}
	else
		bytes = nbytes;

	llsdr.nBytes = htons(bytes) ;

	/* Now compose the block to send to local server */
	lenblock = sizeof(LL_SendDataReq) + ((bytes+ 3) & ~3); 

	/* The extra length to pad it to a multiple of 4 bytes */
	libinfo = (char *)Malloc(lenblock);


	bcopy((char *)&llsdr, libinfo, sizeof(LL_SendDataReq));
	pBuf = libinfo + sizeof(LL_SendDataReq);

	/* Now copy the  data info*/
	bcopy(data, pBuf, bytes);
	pBuf += bytes;

	i = padlength[bytes& 3];
	/* pad */
	while(--i >= 0)
		*pBuf++ = 0;

	/* Now write this to the local server */
	WriteToServer(fd, lenblock, (char *)libinfo);
	
	Free(libinfo) ;

	LEAVE_PROC("LL_SendDataReq");

	return;

}

/*----------------------------------------------LL_SendImageDataReq()---------*/
/*
 *	This is to send a image send request  
 *	information to the local server
 */
void	LL_SendImageDataReq(fd,id,data,nbytes,pkts_left, image_type)
FD		fd;
int		id;
char	*data;
int		nbytes;			/* No of bytes of data */
int		pkts_left;			
unsigned short	image_type ; 
{
	LL_SendDataReq	llsdr;
	int				i,lenblock;
	char			*pBuf;
	char			*libinfo;

	ENTER_PROC("LL_ImageDataReq");

	llsdr.reqType = _ImageDataReq;
	llsdr.gid = id;


	llsdr.nBytes = htons(nbytes );
	llsdr.pkts_left = htons(pkts_left  );
	llsdr.image_type = image_type ; /* XBM/ JPEG */

	/* Now compose the block to send to local server */
	lenblock = sizeof(LL_SendDataReq) + ((nbytes+ 3) & ~3); 

	/* The extra length to pad it to a multiple of 4 bytes */
	libinfo = (char *)Malloc(lenblock);

	bcopy((char *)&llsdr, libinfo, sizeof(LL_SendDataReq));
	pBuf = libinfo + sizeof(LL_SendDataReq);

	bcopy(data, pBuf, nbytes);
	pBuf += nbytes;

	i = padlength[nbytes& 3];
	/* pad */
	while(--i >= 0)
		*pBuf++ = 0;

	/* Now write this to the local server */
	WriteToServer(fd, lenblock, (char *)libinfo);

	LEAVE_PROC("LL_SendImageDataReq");

	return;
}


/*---------------------------------------------LL_SendAudioStartReq()---------*/
/*
 *	This is to send a voice send request  
 *	information to the local server
 */
void	LL_SendAudioStartReq(fd,id)
FD	fd;
int	id;
{
	LL_SendDataReq llsdr;
	int	i,lenblock ;
	char	*pBuf;
	char	*libinfo;

	/* Send request for audio connection */
	llsdr.reqType = _AudioStartReq;
	llsdr.gid = id;

	/* Now compose the block to send to local server */

	lenblock = sizeof(LL_SendDataReq) ;
	libinfo = (char *)Malloc(lenblock);
	bcopy((char *)&llsdr, libinfo, sizeof(LL_SendDataReq));

	/* Now write this to the local server */
	WriteToServer(fd, lenblock, (char *)libinfo);

	return;
}


/*-----------------------------------------------ConnectToServAudio()---------*/
FD	ConnectToServAudio( aport)
int	aport ;
{

	FD	sock ;
	struct	sockaddr_in	aserver ;
	char	*host_name ;
	struct	hostent	*hp ;

/*WORKHERE*//* Should use the server address got form first connect */
	/* Get the host name */
	if ((host_name = getenv ("CONFSERVER")) == (char *)NULL)
		FATAL("No hostname for conference server") ;/*WORKHERE*/ 

	if ((hp = gethostbyname(host_name)) == NULL)
	{
		perror("Invalid host") ;
		exit(-1) ;
	}
	
    if((sock = socket( AF_INET, SOCK_STREAM, 0)) < 0 )
    {
        perror("Opening Stream Socket \n") ;
        return(-1) ;
    }
    aserver.sin_family  = AF_INET ;
	bcopy((char *)hp->h_addr, (char *)&aserver.sin_addr, hp->h_length) ;
    aserver.sin_port    = aport ;

    if ( connect( sock, (struct sockaddr *)&aserver, sizeof( struct sockaddr_in)) <0 )
    {  
        perror("Couldnt connect to img socket\n") ;
        close (sock) ;
        return(-1) ;
    }

	return(sock) ;

}	


/*-----------------------------------------------ConnectToServImage()---------*/
FD	ConnectToServImage( iport)
int	iport ;
{

	FD	sock ;
	struct	sockaddr_in	aserver ;
	char	*host_name ;
	struct	hostent	*hp ;

/*WORKHERE*//* Should use the server address got form first connect */
	/* Get the host name */
	if ((host_name = getenv ("CONFSERVER")) == (char *)NULL)
		FATAL("No hostname for conference server") ;/*WORKHERE*/ 

	if ((hp = gethostbyname(host_name)) == NULL)
	{
		perror("Invalid host") ;
		exit(-1) ;
	}
	
    if((sock = socket( AF_INET, SOCK_STREAM, 0)) < 0 )
    {
        perror("Opening Stream Socket \n") ;
        return(-1) ;
    }
    aserver.sin_family  = AF_INET ;
	bcopy((char *)hp->h_addr, (char *)&aserver.sin_addr, hp->h_length) ;
    aserver.sin_port    = iport ;

    if ( connect( sock, (struct sockaddr *)&aserver, 
								sizeof( struct sockaddr_in)) <0 )
    {  
        perror("Couldnt connect to img socket\n") ;
        close (sock) ;
        return(-1) ;
    }

	return(sock) ;

}	

/*-------------------------------------------------LL_SendAudioData()---------*/
/*
 *	This is to send a audio data send request  
 *	information to the server
 */
void	LL_SendAudioData(fd,id,data,nbytes)
FD		fd;
int		id;
char	*data;
int		nbytes;			/* No of bytes of data */
{
	LL_SendDataReq	llsdr;
	int				i,lenblock;
	char			*pBuf;
	char			*libinfo;

	ENTER_PROC("LL_AudioDataReq");

	llsdr.reqType = _AudioDataReq;
	llsdr.gid = id;

	if( nbytes != strlen(data))
	{
		ERROR("No of bytes passed is wrong:Taking min value");
		DEBUG_LEVEL3((stderr,"strlen retd:%d\tnbytes passed:%d\n",
						strlen(data), nbytes));
		llsdr.nBytes = (strlen(data) < nbytes )? strlen(data):nbytes;
	}
	else
		llsdr.nBytes = nbytes;

	llsdr.nBytes = nbytes ;

	/* Now compose the block to send to local server */

	lenblock = sizeof(LL_SendDataReq) + 
			((llsdr.nBytes+ 3) & ~3); 

	/* The extra length to pad it to a multiple of 4 bytes */

	libinfo = (char *)Malloc(lenblock);

	bcopy((char *)&llsdr, libinfo, sizeof(LL_SendDataReq));
	pBuf = libinfo + sizeof(LL_SendDataReq);

	/* Now copy the alias data info*/
	bcopy(data, pBuf, llsdr.nBytes);
	pBuf += llsdr.nBytes;

	i = padlength[llsdr.nBytes& 3];
	/* pad */
	while(--i >= 0)
		*pBuf++ = 0;

	/* Now write this to the local server */
	WriteToServer(fd, lenblock, (char *)libinfo);

	LEAVE_PROC("LL_SendDataReq");

	return;

}

/*----------------------------------------------LL_SendAudioStopReq()---------*/
/*
 *	This is to send a audio stop request  
 *	information to the server
 */
void	LL_SendAudioStopReq(fd,id)
FD		fd;
int		id;
{
	LL_SendDataReq	llsdr;
	int				lenblock ;
	char			*libinfo;

	/* Send request for audio connection */
	llsdr.reqType = _AudioStopReq;
	llsdr.gid = id;

	/* Now compose the block to send to local server */

	lenblock = sizeof(LL_SendDataReq) ;
	libinfo = (char *)Malloc(lenblock);
	bcopy((char *)&llsdr, libinfo, sizeof(LL_SendDataReq));

	/* Now write this to the local server */
	WriteToServer(fd, lenblock, (char *)libinfo);

	return;
}

/*---------------------------------------------LL_SendImageStartReq()---------*/
void	LL_SendImageStartReq( fd, id)
FD		fd ;
int		id ;
{
	LL_SendDataReq	llsdr ;
	int				i, lenblock ;
	char			*libinfo;

	/* Send request for image connection */

	llsdr.reqType = _ImageStartReq ;
	llsdr.gid = id ;
	
	/* Compose the blokc to send */
	lenblock = sizeof(LL_SendDataReq) ;
	libinfo = (char *)Malloc(lenblock) ;
	bcopy((char *)&llsdr, libinfo, sizeof(LL_SendDataReq)) ;
	WriteToServer(fd, lenblock, (char *)libinfo) ;

	return ;
}

/*----------------------------------------------LL_SendBBoardObjReq()---------*/
/*
 *	This is to send a  request  for drawing an object in blackboard
 */
void	LL_SendBBoardObjReq(fd,id,lstyle, fstyle, lwidth, color, type,data,nbytes)
FD		fd;
int		id;
int		lstyle, fstyle, lwidth ;
int		color ;
int		type ;
int		*data;
int		nbytes;			/* No of bytes of data */
{
	LL_BBoardObjReq llsbor;
	int				i,lenblock;
	char			*pBuf;
	char			*libinfo;

	ENTER_PROC("LL_SendBBoardObjReq");

	llsbor.reqType = _BBoardObjReq;
	llsbor.obj_type = type;
	llsbor.gid = id;
	llsbor.lstyle = lstyle ;
	llsbor.color = color ;
	llsbor.fstyle = fstyle ;
	llsbor.lwidth = htons(lwidth );

	llsbor.nBytes = htons(nbytes);

	/* Now compose the block to send to local server */
	lenblock = sizeof(LL_SendDataReq) + ((nbytes+ 3) & ~3); 

	/* The extra length to pad it to a multiple of 4 bytes */
	libinfo = (char *)Malloc(lenblock);

	bcopy((char *)&llsbor, libinfo, sizeof(LL_SendDataReq));
	pBuf = libinfo + sizeof(LL_SendDataReq);

	/* Now copy the  data info*/
	bcopy(data, pBuf, nbytes);
	pBuf += nbytes;

	i = padlength[nbytes& 3];
	/* pad */
	while(--i >= 0)
		*pBuf++ = 0;

	/* Now write this to the local server */
	WriteToServer(fd, lenblock, (char *)libinfo);

	Free(libinfo) ;
	LEAVE_PROC("LL_SendBBoardObjReq");

	return;

}
/*---------------------------------------------LL_SendBBoardCharReq()---------*/
/*
 *	This is to send a  request  for drawing a letter in blackboard
 */
void	LL_SendBBoardCharReq(fd,id,fstyle, ffam, fsz, letter,color,x,y)
FD		fd;
int		id;
int		fstyle, ffam, fsz ;
int		color;
char	letter ;
short	x,y ;
{
	LL_BBoardCharReq	llsbcr;
	int					i,lenblock;
	char				*libinfo;

	ENTER_PROC("LL_SendBBoardCharReq");

	llsbcr.reqType = _BBoardCharReq;
	llsbcr.letter = letter;
	llsbcr.gid = id;
	llsbcr.font_family = ffam ;
	llsbcr.font_style = fstyle ;
	llsbcr.font_size = fsz ;
	llsbcr.color = color ;
	llsbcr.pos_x = htons(x);
	llsbcr.pos_y = htons(y);

	/* Now compose the block to send to local server */
	lenblock = sizeof(LL_BBoardCharReq)  ;

	libinfo = (char *)Malloc(lenblock);

	bcopy((char *)&llsbcr, libinfo, sizeof(LL_BBoardCharReq));

	/* Now write this to the local server */
	WriteToServer(fd, lenblock, (char *)libinfo);

	Free(libinfo) ;
	LEAVE_PROC("LL_SendBBoardCharReq");

	return;
}
/*---------------------------------------------End of libio.c-----------------*/
