/*-----------------------------------------------------misc.c-----------------*/
/******************************************************************************/
/*                                                                            */
/*                                                                            */
/*                                Vartalaap                                   */
/*----------------------------------------------------------------------------*/
/*                     Multimedia Conferencing Software                       */
/*                                                                            */
/*                                                                            */
/*        Written by:                                                         */
/*                        Yezdezard Lashkari                                  */
/*                        Viswanath Ramachandran                              */
/*                        Sanjay Malpani                                      */
/*                    and                                                     */
/*                        Vijay Talati                                        */
/*                                                                            */
/*                                                                            */
/*                Department of Computer Science and Engineering              */
/*                Indian Institute of Technology, Bombay 400076.              */
/*                                                                            */
/******************************************************************************/

#include	<signal.h>
#include	"../common/inc.h"
#include	"ex_glbls.h"
#include	"ex_routines.h"
#include	"fncproto.h"
#include	"localrout.h"

/*-----------------------------------------------------Usage()----------------*/
/* Display the usage of the server
 * Called when server called with wrong arguments
 */
/*----------------------------------------------------------------------------*/
void	Usage()
{

	fprintf(stderr, "Usage: local \n");
	fprintf(stderr, "		[-d<debug-level>]\n");
	fprintf(stderr, "		[-i<image-port>]\n");
	fprintf(stderr, "		[-p<input-port>]\n");
	fprintf(stderr, "		[-n<server-name>]\n");

	exit(1);

}

/*--------------------------------------------------ScanArgs()----------------*/
/* Routine to scan the commandline arguments passed to server
 */
/*----------------------------------------------------------------------------*/
void	ScanArgs(argc,argv)
int	argc;
char	**argv;
{
	char	buf[MAXNAMESZ];
	int		level ;

	ENTER_PROC("ScanArgs");

	DebuggingFlag	= 0 ;
	/* Scan Argument list */

	while(--argc > 0)
	{
		++argv;
		if(**argv == '-')
			switch(*++*argv)
			{
				case 'd':
					level = atoi(++*argv) ;
					if (level == 3)
						DebuggingFlag = DL3 ;
					else if ( level == 2)
						DebuggingFlag = DL2 ;
					else
						DebuggingFlag = DL1 ;	
					break ;

			   case 'p':
				InputPort = atoi(++*argv);
				if( InputPort < 0)
					InputPort = 0;
				DEBUG_LEVEL1((stderr,"InputPort:%d\n",InputPort));
				break;

			   case 'i':
				ImagePort = atoi(++*argv);
				if( ImagePort < 0)
					ImagePort = 0;
				DEBUG_LEVEL1((stderr,"ImagePort:%d\n",ImagePort));
				break;

			   case 'n':
				if(++*argv != NULL && **argv != '\0')
				strcpy(buf, *argv);

				DEBUG_LEVEL1((stderr,"LocalWellKnownPortName:%s\n",buf));
				LocalWellKnownName = (char*)
					Malloc((strlen(buf)+2) * sizeof(char));
				strcpy(LocalWellKnownName, buf);
				break;

			   default:
				fprintf(stderr,"Unknown Option %c\n", **argv);
				Usage();
				break;

			   }
		else
			/*
			Usage();*/
			;

		}		/* End of while */

}

/*-----------------------------------------------------AddFD()----------------*/
/* Add the given FD to the list to
 * Descriptors on whihc to do a select
/*----------------------------------------------------------------------------*/
void	AddFD(fd)
FD	fd;
{	
	ENTER_PROC("AddFD");

	if(fd < 0)
	{
		ERROR("Negative FD");
		return ;
	}
	/* else */

	FD_SET(fd,&ReadDescriptors);
	nFDsInUse ++;

	if(fd > HighestFD)
		HighestFD = fd;

}

/*--------------------------------------------------RemoveFD()----------------*/
/* Remove the FD from the list of Descriptors
 */
/*----------------------------------------------------------------------------*/
void	RemoveFD(fd)
FD	fd;
{
	ENTER_PROC("RemoveFD");

	if(fd < 0)
	{
		ERROR("Negative FD");
		return ;
	}

	FD_CLR(fd, &ReadDescriptors);
	nFDsInUse--;

	if( fd == HighestFD)
		HighestFD--;	/* Not a very good solution */

/*REMOVED TEMP. due to probs in closing image channel
	close(fd);
*/

}

/*------------------------------------------------CloseDownClient()-----------*/
/*	There is some breakdown in the client.
 *  Remove the client from the lists
 */ 
/*----------------------------------------------------------------------------*/
void	CloseDownClient(fd)
FD	fd;
{

	ENTER_PROC("CloseDownClient");

	RemoveFD(fd);

	clean_rm_proc(fd);
}

/*------------------------------------------------CloseDownServer()-----------*/
/*
 *	Routine to close down server gracefully if some error occurs
 */
/*----------------------------------------------------------------------------*/
void	CloseDownServer()
{
	int	i;

	ENTER_PROC("CloseDownServer");

	/* Remove the temporary files */
/*WORKHERE*/	

	MSG("SHUTTING DOWN LOCAL SERVER -- FATAL CONDITION");
	
	/*MSG("SHUTTING DOWN CONNECTION TO GLOBAL SERVER");*/

	MSG("SHUTTING DOWN WELL KNOWN PORT");

	close(InputPortFD);
	/*close(InputAuPortFD);*/

	MSG("SHUTTING DOWN ALL CLIENT CONNECTIONS");

	for(i = 0 ; i <= HighestFD ; i++)
	{
		if(FD_ISSET(i, &ReadDescriptors))
			close(i);
	}

	/* Remove all temp files */
	for (i=0; i<MAXPARTICIPANTS; i++)
	{
		if (pinfo[i].pfname != NULL)
			unlink(pinfo[i].pfname) ;
	}

	 MSG("ALL FDs CLOSED. EXITING.....");
	 exit(0);

}

/*------------------------------------------------------SignalINT()-----------*/
/*
 *	Specific signal handling routines
 */
/*----------------------------------------------------------------------------*/
void	SignalINT()
{
	DEBUG_LEVEL1((stderr,"==> SIGINT received"));
	CloseDownServer();
	exit(1);
}

/*-----------------------------------------------------SignalQUIT()-----------*/
void	SignalQUIT()
{
	DEBUG_LEVEL1((stderr,"==> SIGQUIT received"));
	CloseDownServer();
	exit(1);
}

/*-----------------------------------------------------SignalTERM()-----------*/
void	SignalTERM()
{
	DEBUG_LEVEL1((stderr,"==> SIGTERM received"));
	CloseDownServer();
	exit(1);
}

/*------------------------------------------------SetLocalSigHandlers()-------*/
/*
 *	Set Local Signal Handling
 */
/*----------------------------------------------------------------------------*/
void	SetLocalSigHandlers()
{
	
	ENTER_PROC("SetLocalSigHandlers");

	signal(SIGINT,SignalINT);
	signal(SIGQUIT,SignalQUIT);
	signal(SIGTERM,SignalTERM);

}




/*-------------------------------------------------DetermineByteOrder()-------*/
/*  To determine byte order
 *  Whether big endian or little endian
 */
/*----------------------------------------------------------------------------*/
void    DetermineByteOrder()
{
    int endian;
    char    byteorder;

    ENTER_PROC("DetermineByteOrder");

    endian = 1;
    if(*(char *)&endian)
        byteorder = 'l';
    else
        byteorder = 'B';

    DEBUG_LEVEL1((stderr, "HOST BYTE ORDER:%c\n", byteorder));
}

/*------------------------------------------------------GetServerPort()-------*/
/* Get the port number for server
 * from base and offset
 */
/*----------------------------------------------------------------------------*/
int	GetServerPort()
{
    int   globport;

    globport = (short )(SERVER_BASE_PORT + InputPort) ;
    DEBUG_LEVEL1((stderr,"Global port no:%d\n", globport));

    return(globport);
}
/*-----------------------------------------------End of misc.c----------------*/


